"""Parser for Crypto PnL (pnl.json)."""
from ..model import UnifiedTrade, BotSummary


def parse_crypto_pnl(data):
    """Parse crypto PnL trades.

    Schema: {trades: [{id, orderId, time, side, price, qty}],
             summary: {total_pnl, ...}}
    """
    trades = []
    raw = data.get("trades", [])

    # Match buy/sell pairs into round-trip trades
    buys = [t for t in raw if t.get("side", "").upper() == "BUY"]
    sells = [t for t in raw if t.get("side", "").upper() == "SELL"]

    for i, sell in enumerate(sells):
        buy = buys[i] if i < len(buys) else None
        if buy:
            buy_price = float(buy.get("price", 0))
            sell_price = float(sell.get("price", 0))
            qty = float(sell.get("qty", 0))
            if buy_price > 0:
                pnl_pct = ((sell_price - buy_price) / buy_price) * 100
            else:
                pnl_pct = 0
            trades.append(UnifiedTrade(
                bot="Crypto (MEXC)",
                asset="XRP/USDT",
                direction="LONG",
                entry_price=buy_price,
                exit_price=sell_price,
                pnl_pct=pnl_pct,
                entry_time=buy.get("time", ""),
                exit_time=sell.get("time", ""),
            ))

    return BotSummary(name="Crypto (MEXC)", trades=trades)



"""Bot-specific JSON parsers. Each parser normalizes data into UnifiedTrade format."""
from .hf_bot import parse_hf_bot
from .integrated_bot import parse_integrated_bot
from .stock_bot import parse_stock_bot
from .crypto import parse_crypto_pnl

PARSERS = {
    "hf_evolution_state.json": parse_hf_bot,
    "paper_trades.json": parse_integrated_bot,
    "stock_paper_trades.json": parse_stock_bot,
    "pnl.json": parse_crypto_pnl,
}



"""Tests for trade_panorama.parsers."""
import unittest
from trade_panorama.parsers.crypto import parse_crypto_pnl
from trade_panorama.parsers.integrated_bot import parse_integrated_bot
from trade_panorama.parsers.stock_bot import parse_stock_bot
from trade_panorama.parsers.hf_bot import parse_hf_bot


class TestCryptoParser(unittest.TestCase):
    def test_basic_pair(self):
        data = {"trades": [
            {"side": "BUY", "price": "100", "qty": "1", "time": "2026-01-01"},
            {"side": "SELL", "price": "110", "qty": "1", "time": "2026-01-02"},
        ]}
        s = parse_crypto_pnl(data)
        self.assertEqual(s.name, "Crypto (MEXC)")
        self.assertEqual(s.total_trades, 1)
        self.assertAlmostEqual(s.trades[0].pnl_pct, 10.0)

    def test_empty(self):
        s = parse_crypto_pnl({"trades": []})
        self.assertEqual(s.total_trades, 0)

    def test_no_trades_key(self):
        s = parse_crypto_pnl({})
        self.assertEqual(s.total_trades, 0)

    def test_multiple_pairs(self):
        data = {"trades": [
            {"side": "BUY", "price": "200", "qty": "1", "time": ""},
            {"side": "BUY", "price": "300", "qty": "1", "time": ""},
            {"side": "SELL", "price": "220", "qty": "1", "time": ""},
            {"side": "SELL", "price": "280", "qty": "1", "time": ""},
        ]}
        s = parse_crypto_pnl(data)
        self.assertEqual(s.total_trades, 2)


class TestIntegratedBotParser(unittest.TestCase):
    def test_full(self):
        data = {
            "capital": 900,
            "initial_capital": 1000,
            "closed_positions": [
                {"asset": "BTC", "direction": "LONG", "entry_price": 100,
                 "exit_price": 95, "pnl_pct": -5, "entry_time": "",
                 "exit_time": "", "strategy": "mean_rev"},
            ],
            "open_positions": [
                {"asset": "ETH", "direction": "LONG", "entry_price": 50,
                 "entry_time": "2026-01-01", "size_usd": 200, "strategy": "trend"},
            ],
        }
        s = parse_integrated_bot(data)
        self.assertEqual(s.name, "Integrated Bot")
        self.assertEqual(s.total_trades, 1)
        self.assertEqual(len(s.open_positions), 1)
        self.assertEqual(s.capital, 900)
        self.assertAlmostEqual(s.capital_pnl, -100)

    def test_empty(self):
        s = parse_integrated_bot({})
        self.assertEqual(s.total_trades, 0)
        self.assertEqual(len(s.open_positions), 0)


class TestStockBotParser(unittest.TestCase):
    def test_uses_symbol(self):
        data = {
            "capital": 5000,
            "initial_capital": 5000,
            "closed_positions": [
                {"symbol": "AAPL", "direction": "LONG", "entry_price": 150,
                 "exit_price": 160, "pnl_pct": 6.67, "entry_time": "",
                 "exit_time": "", "strategy": "breakout"},
            ],
            "open_positions": [],
        }
        s = parse_stock_bot(data)
        self.assertEqual(s.name, "Stock Bot")
        self.assertEqual(s.trades[0].asset, "AAPL")


class TestHfBotParser(unittest.TestCase):
    def test_population(self):
        data = {"population": [
            {
                "id": "V001",
                "strategy_type": "momentum",
                "trades": [
                    {"asset": "BTC", "direction": "LONG", "entry_price": 100,
                     "exit_price": 105, "pnl_pct": 5, "entry_time": "",
                     "exit_time": ""},
                    {"asset": "ETH", "direction": "SHORT", "entry_price": 50,
                     "exit_price": 48, "pnl_pct": 4, "entry_time": "",
                     "exit_time": ""},
                ],
            },
            {
                "id": "V002",
                "strategy_type": "range",
                "trades": [
                    {"asset": "BTC", "direction": "SHORT", "entry_price": 100,
                     "exit_price": 102, "pnl_pct": -2, "entry_time": "",
                     "exit_time": ""},
                ],
            },
        ]}
        s = parse_hf_bot(data)
        self.assertEqual(s.name, "HF Bot")
        self.assertEqual(s.total_trades, 3)
        self.assertEqual(s.wins, 2)
        self.assertEqual(s.trades[0].strategy, "momentum:V001")

    def test_empty_population(self):
        s = parse_hf_bot({"population": []})
        self.assertEqual(s.total_trades, 0)


if __name__ == "__main__":
    unittest.main()



"""Scan directories for trading bot data files and aggregate."""
import json
import os
from .model import BotSummary
from .parsers import PARSERS


def find_data_files(root_dir):
    """Walk directory tree looking for known bot data files."""
    found = {}
    for dirpath, _dirnames, filenames in os.walk(root_dir):
        for fname in filenames:
            if fname in PARSERS:
                full = os.path.join(dirpath, fname)
                # Avoid duplicates: keep deepest path
                if fname not in found or len(full) > len(found[fname]):
                    found[fname] = full
    return found


def scan(root_dir, files=None):
    """Scan for all bot data and return list of BotSummary.

    Args:
        root_dir: Root directory to search for data files
        files: Optional dict of {filename: path} to use instead of scanning
    """
    if files is None:
        files = find_data_files(root_dir)

    summaries = []
    for fname, fpath in files.items():
        parser = PARSERS.get(fname)
        if not parser:
            continue
        try:
            with open(fpath) as f:
                data = json.load(f)
            summary = parser(data)
            summaries.append(summary)
        except (json.JSONDecodeError, KeyError, TypeError) as e:
            print("WARNING: Failed to parse {}: {}".format(fpath, e))

    return summaries


def aggregate(summaries):
    """Compute cross-bot aggregate stats."""
    total_trades = sum(s.total_trades for s in summaries)
    total_wins = sum(s.wins for s in summaries)
    total_pnl_pct = sum(s.total_pnl_pct for s in summaries)
    total_open = sum(len(s.open_positions) for s in summaries)
    total_capital = sum(s.capital for s in summaries)
    total_initial = sum(s.initial_capital for s in summaries)

    return {
        "bots": len(summaries),
        "total_trades": total_trades,
        "total_wins": total_wins,
        "win_rate": total_wins / total_trades if total_trades > 0 else 0,
        "total_pnl_pct": total_pnl_pct,
        "open_positions": total_open,
        "total_capital": total_capital,
        "total_initial_capital": total_initial,
        "capital_pnl": total_capital - total_initial if total_initial > 0 else 0,
    }



"""Unified data model for cross-bot trade aggregation."""


class UnifiedTrade:
    """A single trade normalized from any bot's JSON format."""

    def __init__(self, bot, asset, direction, entry_price, exit_price,
                 pnl_pct, entry_time, exit_time, strategy=""):
        self.bot = bot
        self.asset = asset
        self.direction = direction
        self.entry_price = entry_price
        self.exit_price = exit_price
        self.pnl_pct = pnl_pct
        self.entry_time = entry_time
        self.exit_time = exit_time
        self.strategy = strategy

    def to_dict(self):
        return {
            "bot": self.bot,
            "asset": self.asset,
            "direction": self.direction,
            "entry_price": self.entry_price,
            "exit_price": self.exit_price,
            "pnl_pct": self.pnl_pct,
            "entry_time": self.entry_time,
            "exit_time": self.exit_time,
            "strategy": self.strategy,
        }


class OpenPosition:
    """An open (unclosed) position."""

    def __init__(self, bot, asset, direction, entry_price, entry_time,
                 size_usd=0, strategy=""):
        self.bot = bot
        self.asset = asset
        self.direction = direction
        self.entry_price = entry_price
        self.entry_time = entry_time
        self.size_usd = size_usd
        self.strategy = strategy

    def to_dict(self):
        return {
            "bot": self.bot,
            "asset": self.asset,
            "direction": self.direction,
            "entry_price": self.entry_price,
            "entry_time": self.entry_time,
            "size_usd": self.size_usd,
            "strategy": self.strategy,
        }


class BotSummary:
    """Aggregated stats for one bot."""

    def __init__(self, name, trades=None, open_positions=None,
                 capital=0, initial_capital=0):
        self.name = name
        self.trades = trades or []
        self.open_positions = open_positions or []
        self.capital = capital
        self.initial_capital = initial_capital

    @property
    def total_trades(self):
        return len(self.trades)

    @property
    def wins(self):
        return sum(1 for t in self.trades if t.pnl_pct > 0)

    @property
    def win_rate(self):
        if not self.trades:
            return 0.0
        return self.wins / len(self.trades)

    @property
    def total_pnl_pct(self):
        return sum(t.pnl_pct for t in self.trades)

    @property
    def capital_pnl(self):
        if self.initial_capital > 0:
            return self.capital - self.initial_capital
        return 0.0

    def to_dict(self):
        return {
            "name": self.name,
            "total_trades": self.total_trades,
            "wins": self.wins,
            "win_rate": self.win_rate,
            "total_pnl_pct": self.total_pnl_pct,
            "open_positions": len(self.open_positions),
            "capital": self.capital,
            "initial_capital": self.initial_capital,
            "capital_pnl": self.capital_pnl,
        }



"""Tests for trade_panorama.model."""
import unittest
from trade_panorama.model import UnifiedTrade, OpenPosition, BotSummary


class TestUnifiedTrade(unittest.TestCase):
    def test_to_dict(self):
        t = UnifiedTrade("Bot", "BTC", "LONG", 100, 110, 10.0,
                         "2026-01-01", "2026-01-02", "momentum")
        d = t.to_dict()
        self.assertEqual(d["bot"], "Bot")
        self.assertEqual(d["asset"], "BTC")
        self.assertEqual(d["pnl_pct"], 10.0)
        self.assertEqual(d["strategy"], "momentum")

    def test_default_strategy(self):
        t = UnifiedTrade("Bot", "ETH", "SHORT", 50, 45, 10.0, "", "")
        self.assertEqual(t.strategy, "")


class TestOpenPosition(unittest.TestCase):
    def test_to_dict(self):
        p = OpenPosition("Bot", "AAPL", "LONG", 150.0, "2026-01-01",
                         size_usd=1000, strategy="breakout")
        d = p.to_dict()
        self.assertEqual(d["size_usd"], 1000)
        self.assertEqual(d["strategy"], "breakout")


class TestBotSummary(unittest.TestCase):
    def _make_trade(self, pnl):
        return UnifiedTrade("B", "X", "LONG", 100, 100 + pnl, pnl, "", "")

    def test_empty(self):
        s = BotSummary("Test")
        self.assertEqual(s.total_trades, 0)
        self.assertEqual(s.wins, 0)
        self.assertAlmostEqual(s.win_rate, 0.0)
        self.assertAlmostEqual(s.total_pnl_pct, 0.0)

    def test_win_rate(self):
        trades = [self._make_trade(5), self._make_trade(-2), self._make_trade(3)]
        s = BotSummary("Test", trades=trades)
        self.assertEqual(s.total_trades, 3)
        self.assertEqual(s.wins, 2)
        self.assertAlmostEqual(s.win_rate, 2 / 3)

    def test_total_pnl(self):
        trades = [self._make_trade(5), self._make_trade(-3)]
        s = BotSummary("Test", trades=trades)
        self.assertAlmostEqual(s.total_pnl_pct, 2.0)

    def test_capital_pnl(self):
        s = BotSummary("Test", capital=1200, initial_capital=1000)
        self.assertAlmostEqual(s.capital_pnl, 200)

    def test_capital_pnl_no_initial(self):
        s = BotSummary("Test", capital=500, initial_capital=0)
        self.assertAlmostEqual(s.capital_pnl, 0.0)

    def test_to_dict(self):
        trades = [self._make_trade(10)]
        pos = [OpenPosition("B", "Y", "SHORT", 50, "")]
        s = BotSummary("Test", trades=trades, open_positions=pos,
                       capital=1100, initial_capital=1000)
        d = s.to_dict()
        self.assertEqual(d["name"], "Test")
        self.assertEqual(d["total_trades"], 1)
        self.assertEqual(d["open_positions"], 1)
        self.assertEqual(d["capital_pnl"], 100)


if __name__ == "__main__":
    unittest.main()



"""Tests for trade_panorama.scanner."""
import json
import os
import tempfile
import unittest
from trade_panorama.scanner import find_data_files, scan, aggregate
from trade_panorama.model import UnifiedTrade, BotSummary


class TestFindDataFiles(unittest.TestCase):
    def test_finds_known_files(self):
        with tempfile.TemporaryDirectory() as d:
            # Create nested structure
            sub = os.path.join(d, "bot1", "data")
            os.makedirs(sub)
            path = os.path.join(sub, "pnl.json")
            with open(path, "w") as f:
                json.dump({"trades": []}, f)

            found = find_data_files(d)
            self.assertIn("pnl.json", found)
            self.assertEqual(found["pnl.json"], path)

    def test_ignores_unknown(self):
        with tempfile.TemporaryDirectory() as d:
            with open(os.path.join(d, "random.json"), "w") as f:
                json.dump({}, f)
            found = find_data_files(d)
            self.assertEqual(len(found), 0)

    def test_keeps_deepest(self):
        with tempfile.TemporaryDirectory() as d:
            shallow = os.path.join(d, "pnl.json")
            deep_dir = os.path.join(d, "a", "b")
            os.makedirs(deep_dir)
            deep = os.path.join(deep_dir, "pnl.json")

            for p in [shallow, deep]:
                with open(p, "w") as f:
                    json.dump({"trades": []}, f)

            found = find_data_files(d)
            self.assertEqual(found["pnl.json"], deep)


class TestScan(unittest.TestCase):
    def test_scan_with_files(self):
        with tempfile.TemporaryDirectory() as d:
            path = os.path.join(d, "pnl.json")
            with open(path, "w") as f:
                json.dump({"trades": []}, f)

            summaries = scan(d, files={"pnl.json": path})
            self.assertEqual(len(summaries), 1)
            self.assertEqual(summaries[0].name, "Crypto (MEXC)")

    def test_scan_bad_json(self):
        with tempfile.TemporaryDirectory() as d:
            path = os.path.join(d, "pnl.json")
            with open(path, "w") as f:
                f.write("not json")
            summaries = scan(d, files={"pnl.json": path})
            self.assertEqual(len(summaries), 0)


class TestAggregate(unittest.TestCase):
    def _make(self, name, pnls, capital=0, initial=0, n_open=0):
        trades = [UnifiedTrade(name, "X", "LONG", 100, 100 + p, p, "", "")
                  for p in pnls]
        from trade_panorama.model import OpenPosition
        opens = [OpenPosition(name, "Y", "LONG", 50, "") for _ in range(n_open)]
        return BotSummary(name, trades=trades, open_positions=opens,
                          capital=capital, initial_capital=initial)

    def test_aggregate(self):
        s1 = self._make("A", [5, -2, 3], capital=1100, initial=1000, n_open=1)
        s2 = self._make("B", [-1, 4], capital=500, initial=500, n_open=2)
        agg = aggregate([s1, s2])
        self.assertEqual(agg["bots"], 2)
        self.assertEqual(agg["total_trades"], 5)
        self.assertEqual(agg["total_wins"], 3)
        self.assertAlmostEqual(agg["win_rate"], 3 / 5)
        self.assertAlmostEqual(agg["total_pnl_pct"], 9.0)
        self.assertEqual(agg["open_positions"], 3)
        self.assertEqual(agg["total_capital"], 1600)
        self.assertAlmostEqual(agg["capital_pnl"], 100)

    def test_aggregate_empty(self):
        agg = aggregate([])
        self.assertEqual(agg["total_trades"], 0)
        self.assertEqual(agg["win_rate"], 0)


if __name__ == "__main__":
    unittest.main()



"""Parser for Stock Paper Trade Bot (stock_paper_trades.json)."""
from ..model import UnifiedTrade, OpenPosition, BotSummary


def parse_stock_bot(data):
    """Parse stock bot paper trades.

    Schema: {capital, initial_capital, open_positions: [{symbol, strategy,
             direction, entry_price, entry_time, size_usd}],
             closed_positions: [{symbol, strategy, direction, entry_price,
             exit_price, pnl_pct, ...}]}
    """
    trades = []
    for t in data.get("closed_positions", []):
        trades.append(UnifiedTrade(
            bot="Stock Bot",
            asset=t.get("symbol", ""),
            direction=t.get("direction", ""),
            entry_price=t.get("entry_price", 0),
            exit_price=t.get("exit_price", 0),
            pnl_pct=t.get("pnl_pct", 0),
            entry_time=t.get("entry_time", ""),
            exit_time=t.get("exit_time", ""),
            strategy=t.get("strategy", ""),
        ))

    open_pos = []
    for p in data.get("open_positions", []):
        open_pos.append(OpenPosition(
            bot="Stock Bot",
            asset=p.get("symbol", ""),
            direction=p.get("direction", ""),
            entry_price=p.get("entry_price", 0),
            entry_time=p.get("entry_time", ""),
            size_usd=p.get("size_usd", 0),
            strategy=p.get("strategy", ""),
        ))

    return BotSummary(
        name="Stock Bot",
        trades=trades,
        open_positions=open_pos,
        capital=data.get("capital", 0),
        initial_capital=data.get("initial_capital", 0),
    )



"""Parser for HF Evolution Engine (hf_evolution_state.json)."""
from ..model import UnifiedTrade, OpenPosition, BotSummary


def parse_hf_bot(data):
    """Parse HF evolution state into BotSummary.

    Schema: {population: [{id, strategy_type, trades: [{asset, direction,
             entry_price, exit_price, pnl_pct, entry_time, exit_time}]}]}
    """
    trades = []
    for variant in data.get("population", []):
        strategy = variant.get("strategy_type", "") + ":" + variant.get("id", "")
        for t in variant.get("trades", []):
            trades.append(UnifiedTrade(
                bot="HF Bot",
                asset=t.get("asset", ""),
                direction=t.get("direction", ""),
                entry_price=t.get("entry_price", 0),
                exit_price=t.get("exit_price", 0),
                pnl_pct=t.get("pnl_pct", 0),
                entry_time=t.get("entry_time", ""),
                exit_time=t.get("exit_time", ""),
                strategy=strategy,
            ))

    return BotSummary(name="HF Bot", trades=trades)



"""CLI interface for trade-panorama."""
import argparse
import json
import os
import sys

from . import __version__
from .scanner import scan, aggregate


# ANSI colors (with fallback)
def _color(code, text, no_color=False):
    if no_color or not sys.stdout.isatty():
        return text
    return "\033[{}m{}\033[0m".format(code, text)


def green(t, nc=False): return _color("32", t, nc)
def red(t, nc=False): return _color("31", t, nc)
def yellow(t, nc=False): return _color("33", t, nc)
def bold(t, nc=False): return _color("1", t, nc)
def dim(t, nc=False): return _color("2", t, nc)


def format_pnl(val, nc=False):
    if val > 0:
        return green("+{:.2f}%".format(val), nc)
    elif val < 0:
        return red("{:.2f}%".format(val), nc)
    return "0.00%"


def format_usd(val, nc=False):
    if val > 0:
        return green("+${:.0f}".format(val), nc)
    elif val < 0:
        return red("-${:.0f}".format(abs(val)), nc)
    return "$0"


def print_summary(summaries, agg, no_color=False):
    nc = no_color
    print()
    print(bold("=== Trade Panorama ===", nc))
    print()

    # Per-bot table
    header = "{:<18} {:>7} {:>7} {:>10} {:>6} {:>12}".format(
        "Bot", "Trades", "WR", "PnL%", "Open", "Capital")
    print(bold(header, nc))
    print("-" * 65)

    for s in summaries:
        wr_str = "{:.0f}%".format(s.win_rate * 100) if s.total_trades > 0 else "-"
        pnl_str = format_pnl(s.total_pnl_pct, nc)
        cap_str = ""
        if s.initial_capital > 0:
            cap_str = "${:.0f}".format(s.capital)
            diff = s.capital_pnl
            if diff != 0:
                cap_str += " ({})".format(format_usd(diff, nc))

        print("{:<18} {:>7} {:>7} {:>10} {:>6} {:>12}".format(
            s.name,
            s.total_trades,
            wr_str,
            pnl_str,
            len(s.open_positions),
            cap_str,
        ))

    print("-" * 65)

    # Aggregate
    agg_wr = "{:.0f}%".format(agg["win_rate"] * 100) if agg["total_trades"] > 0 else "-"
    print("{:<18} {:>7} {:>7} {:>10} {:>6}".format(
        bold("TOTAL", nc),
        agg["total_trades"],
        agg_wr,
        format_pnl(agg["total_pnl_pct"], nc),
        agg["open_positions"],
    ))

    # Open positions detail
    all_open = []
    for s in summaries:
        all_open.extend(s.open_positions)

    if all_open:
        print()
        print(bold("Open Positions ({})".format(len(all_open)), nc))
        print("{:<18} {:<10} {:>5} {:>12} {:>10} {}".format(
            "Bot", "Asset", "Dir", "Entry", "Size", "Since"))
        print("-" * 70)
        for p in all_open:
            since = p.entry_time[:10] if p.entry_time else ""
            print("{:<18} {:<10} {:>5} {:>12.2f} {:>9} {}".format(
                p.bot, p.asset, p.direction,
                p.entry_price,
                "${:.0f}".format(p.size_usd) if p.size_usd else "-",
                since,
            ))

    print()


def main():
    parser = argparse.ArgumentParser(
        prog="trade-panorama",
        description="Unified PnL dashboard across multiple trading bots",
    )
    parser.add_argument("--version", action="version",
                        version="trade-panorama " + __version__)
    parser.add_argument("--dir", "-d", default=".",
                        help="Root directory to scan for bot data (default: .)")
    parser.add_argument("--json", action="store_true",
                        help="Output as JSON")
    parser.add_argument("--no-color", action="store_true",
                        help="Disable colored output")
    parser.add_argument("--bot", "-b", action="append",
                        help="Filter by bot name (can repeat)")

    args = parser.parse_args()
    root = os.path.abspath(args.dir)

    summaries = scan(root)

    if not summaries:
        print("No trading bot data found in: {}".format(root))
        print("Looked for: {}".format(", ".join(sorted(
            __import__("trade_panorama.parsers", fromlist=["PARSERS"]).PARSERS.keys()
        ))))
        sys.exit(1)

    # Filter by bot name
    if args.bot:
        filters = [b.lower() for b in args.bot]
        summaries = [s for s in summaries if
                     any(f in s.name.lower() for f in filters)]

    agg = aggregate(summaries)

    if args.json:
        output = {
            "bots": [s.to_dict() for s in summaries],
            "aggregate": agg,
        }
        print(json.dumps(output, indent=2, ensure_ascii=False))
    else:
        print_summary(summaries, agg, no_color=args.no_color)


if __name__ == "__main__":
    main()
